﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Data;
using HeuristicLab.Encodings.ConditionActionEncoding;
using HeuristicLab.Operators;
using HeuristicLab.Parameters;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;

namespace HeuristicLab.Encodings.CombinedIntegerVectorEncoding {
  [Item("CombinedIntegerVectorCoveringCreator", "Description missing")]
  [StorableClass]
  public class CombinedIntegerVectorCoveringCreator : SingleSuccessorOperator, ICoveringSolutionCreator {

    #region Parameter Properties
    public ILookupParameter<IClassifier> CoverClassifierParameter {
      get { return (ILookupParameter<IClassifier>)Parameters["CoverClassifier"]; }
    }
    public ILookupParameter<IClassifier> ActionParameter {
      get { return (ILookupParameter<IClassifier>)Parameters["Action"]; }
    }
    public IValueLookupParameter<IClassifier> CreatedClassifierParameter {
      get { return (IValueLookupParameter<IClassifier>)Parameters["CreatedClassifier"]; }
    }
    public ILookupParameter<PercentValue> ChangeSymbolProbabilityParameter {
      get { return (ILookupParameter<PercentValue>)Parameters["ChangeSymbolProbability"]; }
    }
    public ILookupParameter<IRandom> RandomParameter {
      get { return (ILookupParameter<IRandom>)Parameters["Random"]; }
    }

    #endregion

    [StorableConstructor]
    protected CombinedIntegerVectorCoveringCreator(bool deserializing) : base(deserializing) { }
    protected CombinedIntegerVectorCoveringCreator(CombinedIntegerVectorCoveringCreator original, Cloner cloner)
      : base(original, cloner) {
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new CombinedIntegerVectorCoveringCreator(this, cloner);
    }
    public CombinedIntegerVectorCoveringCreator()
      : base() {
      Parameters.Add(new LookupParameter<IClassifier>("CoverClassifier"));
      Parameters.Add(new LookupParameter<IClassifier>("Action"));
      Parameters.Add(new ValueLookupParameter<IClassifier>("CreatedClassifier"));
      Parameters.Add(new LookupParameter<PercentValue>("ChangeSymbolProbability"));
      Parameters.Add(new LookupParameter<IRandom>("Random"));
    }

    public override IOperation Apply() {
      CombinedIntegerVector newCondition = (CombinedIntegerVector)CoverClassifierParameter.ActualValue.Condition.Clone();

      CombinedIntegerVector condition = (CombinedIntegerVector)CoverClassifierParameter.ActualValue.Condition;
      CombinedIntegerVector action = (CombinedIntegerVector)ActionParameter.ActualValue.Action;

      newCondition = UniformSomePositionInConditionManipulator.ManipulateCondition(RandomParameter.ActualValue, condition, newCondition, ChangeSymbolProbabilityParameter.ActualValue.Value);

      CreatedClassifierParameter.ActualValue = new CombinedIntegerVector(newCondition, newCondition.Bounds, action, action.Bounds);
      return base.Apply();
    }
  }
}
